#ifndef C804Controller_H
#define C804Controller_H

#include "SINQController.h"
#include "C804Axis.h"
#include "asynMotorAxis.h"

class C804Controller : public SINQController
{
public:
  C804Controller(const char *portName, const char *lowLevelPortName, int lowLevelPortAddress, int numAxes, double movingPollPeriod,
                 double idlePollPeriod, const int &extraParams = 2);

  virtual ~C804Controller();

  /* These are the methods that we override */
  C804Axis *getAxis(asynUser *pasynUser);
  C804Axis *getAxis(int axisNo);
  C804Axis *castToC804Axis(asynMotorAxis *asynAxis);

protected:
  asynUser *lowLevelPortUser_;

  // User-defined polling periods in ms
  time_t movingPollPeriod_;
  time_t idlePollPeriod_;

  void log(const char *message);
  asynStatus lowLevelWriteRead(int axisNo, const char *command, char *response, bool expect_response);

private:
  // Set the maximum buffer size. This is an empirical value which must be large
  // enough to avoid overflows for all commands to the device / responses from it.
  static const uint32_t C804_MAXBUF_ = 200;

  /*
  When trying to communicate with the device, the underlying asynOctetSyncIO
  interface waits for a response until this time (in seconds) has passed,
  then it declares a timeout. This variable has to be specified in the .cpp-file.
  Tying to specify in the .h-file results in the following compiler error:
  In file included from ../sinqEPICSApp/src/C804Axis.cpp:2:
  ../sinqEPICSApp/src/C804Controller.h:38:23: error: ‘constexpr’ needed for in-class initialization
  of static data member ‘const double C804Controller::C804_TIMEOUT_’ of non-integral type
  */
  static const double C804_TIMEOUT_;

  friend class C804Axis;
};

#endif