/*
FILENAME...   EuroMoveDriver.h
USAGE...      Motor driver support for the LLB EuroMove controller

Mark Koennecke
January 2020

*/

#include "SINQController.h"
#include "SINQAxis.h"

#define COMLEN 80

typedef enum {
  HomeIdle,
  HomeFastRun,
  HomeFastStop,
  HomeBackStep,
  HomeSlowRun,
  HomeSlowStop,
} HomingStatus;

class EuroMoveAxis : public SINQAxis
{
public:
  /* These are the methods we override from the base class */
  EuroMoveAxis(class EuroMoveController *pC, int axis);
  void report(FILE *fp, int level);
  asynStatus move(double position, int relative, double min_velocity, double max_velocity, double acceleration);
  asynStatus moveVelocity(double min_velocity, double max_velocity, double acceleration);
  asynStatus home(double min_velocity, double max_velocity, double acceleration, int forwards);
  asynStatus stop(double acceleration);
  asynStatus poll(bool *moving);
  asynStatus setPosition(double position);
  asynStatus setClosedLoop(bool closedLoop);
  void setMotNo(unsigned int no){
    motNo = no;
  }

private:
  EuroMoveController *pC_;          /**< Pointer to the asynMotorController to which this axis belongs.
                                   *   Abbreviated because it is used very frequently */
  time_t next_poll;
  int motNo; // motor number in the EuroMove controller
  int position;
  HomingStatus homingStatus;
  int homingDirection; // 1 = forward, 0 backwards 
  asynStatus sendStop();
  int resolution;
  friend class EuroMoveController;
  int targetPosition;
};

class EuroMoveController : public SINQController {
public:
  EuroMoveController(const char *portName, const char *EuroMovePortName, const int gpibAddr, const int nAxis);

  void report(FILE *fp, int level);
  EuroMoveAxis* getAxis(asynUser *pasynUser);
  EuroMoveAxis* getAxis(int axisNo);
  asynStatus writeFloat64(asynUser *pasynUser, epicsFloat64 value);

friend class EuroMoveAxis;
 private:
 asynUser *pasynUserController_;
 unsigned int gpibAddr;
 char addrCommand[50];

 asynStatus transactController(int axisNo, char command[COMLEN], char reply[COMLEN]);

};
