/*
  This class contains the necessary changes to have an additional text fields for messages
  with each axis.

  Code lifted from Torsten Boegershausen ESS code.

  Mark Koennecke, March 2017

  Added code to manage an interMessageSleep 

  Mark Koennecke, February 2024
*/

#include "SINQController.h"
#include "asynMotorController.h"
#include "epicsExport.h"
#include "iocsh.h"

SINQController::SINQController(const char *portName, const char *SINQPortName, int numAxes, const int& extraParams)
  :  asynMotorController(portName, numAxes+1, NUM_MOTOR_DRIVER_PARAMS+extraParams,
                         0, // No additional interfaces beyond those in base class
                         0, // No additional callback interfaces beyond those in base class
                         ASYN_CANBLOCK | ASYN_MULTIDEVICE, 
                         1, // autoconnect
                         0, 0)  // Default priority and stack size
{
  createParam(motorMessageIsFromDriverString,    asynParamInt32,      &motorMessageIsFromDriver_);
  createParam(motorMessageTextString,            asynParamOctet,      &motorMessageText_);

  interMessageSleep=20;
}

/** Set  the interMessageSleep at runtime */
asynStatus SINQController::setInterMessageSleep(int messageSleep)
{
  lock();
  interMessageSleep = messageSleep;
  unlock();
  return asynSuccess;
}

extern "C" {

asynStatus setInterMessageSleep(const char *portName, int messageSleep)
{
  SINQController *pC;
  static const char *functionName = "setIntermessageSleep";

  pC = (SINQController*) findAsynPortDriver(portName);
  if (!pC) {
    printf("%s:%s: Error port %s not found\n", "SINQController", functionName, portName);
    return asynError;
  }
    
  return pC->setInterMessageSleep(messageSleep);
}


/* setInterMessageSleep */
static const iocshArg setInterMessageSleepArg0 = {"Controller port name", iocshArgString};
static const iocshArg setInterMessageSleepArg1 = {"inter message sleep time", iocshArgInt};
static const iocshArg * const setInterMessageSleepArgs[] = {&setInterMessageSleepArg0,
                                                           &setInterMessageSleepArg1};
static const iocshFuncDef setInterMessageSleepDef = {"setInterMessageSleep", 2, setInterMessageSleepArgs};

static void setInterMessageSleepCallFunc(const iocshArgBuf *args)
{
  setInterMessageSleep(args[0].sval, args[1].ival);
}

static void SINQControllerRegister(void)
{
  iocshRegister(&setInterMessageSleepDef, setInterMessageSleepCallFunc);
}
epicsExportRegistrar(SINQControllerRegister);

} // extern C    

